%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% adddata.m
% 
% Searches the training data for observations which are not observed 
% in that set but are observed in the test data. The function then replaces the last column(s) 
% of the training data with a column containing these new observations.
% This complies with a constraint of the HMM toolbox that all observations
% in the test sequence must be observed in the training sequence. Furthermore, we can only add a vector 
% of values (length of a column), thus previous data from the training set are used to fill the rest of the replacement vector. 
% The data will be somewhat contaminated by this contraint. To avoid contamination one should choose data which comply
% with this constraint. You can try entering a larger percentage of the data set to use as training data.
% 
% Greg Eustace, MUMT 611, final project
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

function [training] =  adddata(training, test1, test2, numobs, numbar, pic);

temp = 0; obstr = 0;

L = length(training);
W = min(size(training)); 

allts = unique([test1 test2]);   % get all observations for both test sets
alltr = unique(training);           % get all observations for training set
 
for i = 1:length(allts),
     fd = find (alltr == allts(i));
     if isempty(fd)
         temp = [temp allts(i)]; % get all observations in test set which are not observed in training set.
     end
end
 
if length(temp) > 1
	temp = temp(2:length(temp));
    lt = length(temp); 
    if numbar == 1
        temp = [temp training(1:W-lt,1)'];
    else
        if pic == 'p'
            temp = [temp training(1:W-lt,16)'];
        else
            temp = [temp training(1:W-lt,15)'];
        end
    end
    
    temp = temp';
    training = [training temp]; %training(:,length(training)) = temp;
   
    disp(' ');
    disp('Replaced some data in training set with observations from test set(s) (see adddata.m for details)');
end
