%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% classify.m
% 
% Classifies melodic data obtained from Chopins Nocturnes and Bachs
% two-part inventions using the HMMtoolbox by Kevin Murphy available at:
% http://www.cs.ubc.ca/~murphyk/Software/HMM/hmm.html
% 
% Greg Eustace, MUMT 611, final project
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;

%--------------Pre-processing values---------------%

L = 32; % length of two bars of 1/16th notes in 4/4
pn = 12; % number of observations for pitch data
in = 24; % number of observation for interval data
cn = 5; % number of observation for contour data
W = 15; % number of pieces in each data set (i.e. number of rows)

%--------------Input parameters--------------------------%

pic = input('Enter melodic representation. p (pitch), i(interval) or c(contour): ', 's');
if (isempty(pic) || pic ~= 'p' && pic ~= 'i' && pic ~= 'c' ), 
    pic = 'c';
    disp('default set (pic = c)');
end
disp(' ');

rhm = input('Add rhythm? y/n: ', 's'); 
if (isempty(rhm) || rhm ~= 'y'), 
    disp('default set (rhm = n)');
    rhm = 'n';
end
disp(' ');

numbar = input('Form the data set from the first one or two bars of each piece. 1 or 2: ');
if (isempty(numbar) || numbar ~= 1 && numbar ~= 2), 
    numbar = 2;
    disp('default set (numbar = 2)');
end
disp(' ');

perc = input('Enter percentage of data set to use as training data (0 to 100): ');
if (isempty(perc) || perc < 0 || perc > 100), 
    perc = 33.3;
    disp('default set (perc = 33.3%)');
end
disp(' ');

type = input('Enter the type of HMM to use.\n 1 = A strict left-right model in which each state can transfer to its self and the next one.\n 2 = left-right model in which each state can transfer to its self and any state to the right of it.\n 3 = A left-right model in which the last state can transfer to the first state.\n 4 = a fully connected model. : ');
if (isempty(type) || type > 0 || type < 4), 
    type = 1;
    disp('default set (type = 1)');
end
disp(' ');

numsts = input('Enter the number of states in the model: ');
if (isempty(numsts) || numsts > 0 || numsts < L), 
    if numbar == 1
        numsts = 16;
    else
        numsts = 32;
    end
    disp('default set (number of 1/16th notes multiplied by number of bars)');
end
disp(' ');

borc = input('Choose an option: \n1 = Train Bach and test Bach and Chopin. \n2 = Train Chopin and test Chopin and Bach: ');
if (isempty(borc) || borc < 0 || borc > 2), 
    borc = 1;
    disp('default set (borc = 1)');
end    
disp(' ');

numitr = input('Enter number of iterations of EM: ');  
if (isempty(numitr) || numitr < 0), 
    numitr = 5;
    disp('default set (numitr = 5)');
end    
disp(' ');

disp('preprocessing...');

%--------------------MIDI note numbers and note durations from Bach Inventions 1-15------------------%
[bp, br] = bach_data(L);

%--------------------MIDI note numbers and note durations from Chopin Nocturnes 1-10, 13-15, 18 and 21------------------%
[cp, cr] = chopin_data(L);

%-----Pitch calculations-----%
if pic == 'p'
	for i=1:W,
        bp(i,:) = normp(bp(i,:)); 
        cp(i,:) = normp(cp(i,:)); 
	end
	
	bp = bp-59; % Adjust pitches to start at 1. 
    cp = cp-59;
end

%-----Interval calculations-----%
if pic == 'i' || pic == 'c'
	for i=1:W,
        bi(i,:) = itv(bp(i,:));
        ci(i,:) = itv(cp(i,:));
	end
	bi = bi(:,2:length(bi)); % discard first interval
    ci = ci(:,2:length(ci));

	for i=1:W,
        bi(i,:) = normi(bi(i,:)); % Normalize intervals to the octave
        ci(i,:) = normi(ci(i,:));
	end
    
    bi = bi+13; % Adjust intervals to start at one 
    ci = ci+13;
        
	%--------Contour calculations-------%
	if pic == 'c'
		for k=1:W,
            bc(k,:) = itoc (bi(k,:));
            cc(k,:) = itoc (ci(k,:));
		end
	end
end 

%--------Add rhythm to pitches--------%
if rhm == 'y' && pic == 'p'
	for i=1:W,
        temp = rhy (bp(i,:), br(i,:));
        tempc = rhy (cp(i,:), cr(i,:));
        bpr(i,:) = temp(1:L); 
        cpr(i,:) = tempc(1:L); 
	end 
end

%--------Add rhythm to intervals--------%
if rhm == 'y' && pic == 'i'
	for i=1:W,
        temp2 = rhy (bi(i,:), br(i,:));
        bir(i,:) = temp2(1:L-1); 
        temp2c = rhy (ci(i,:), cr(i,:));
        cir(i,:) = temp2c(1:L-1); 
	end
end

%--------Add rhythm to contours--------%
if rhm == 'y' && pic == 'c'
	for i=1:W,
        temp3 = rhy (bc(i,:), br(i,:));
        bcr(i,:) = temp3(1:L-1); 
        temp3c = rhy (cc(i,:), cr(i,:));
        ccr(i,:) = temp3c(1:L-1); 
	end
end

%-------Concatenate first bars--------%

if numbar == 1 
    if rhm ~= 'y' && pic == 'p'
	    bp = bp(:,1:L/2);
        cp = cp(:,1:L/2);
    elseif rhm == 'y' && pic == 'p'
	    bpr = bpr(:,1:L/2);
        cpr = cpr(:,1:L/2);
    elseif rhm ~= 'y' && pic == 'i'
	    bi = bi(:,1:L/2);
        ci = ci(:,1:L/2);
    elseif rhm == 'y' && pic == 'i'
	     bir = bir(:,1:L/2);
         cir = cir(:,1:L/2);
    elseif rhm ~= 'y' && pic == 'c',
        bc = bc(:,1:L/2);
        cc = cc(:,1:L/2);
    elseif rhm == 'y' && pic == 'c'
	    bcr = bcr(:,1:L/2);
        ccr = ccr(:,1:L/2);
    end
end

%--------------Separate training and testing data---------%
if rhm ~= 'y' && pic == 'p'
    disp('pitch data');
    [bpTs, bpTr] = separate(bp, perc, W); % Separate training and testing data
    [cpTs, cpTr] = separate(cp, perc, W);
    
    bpTr = adddata(bpTr, bpTs, cpTs, pn, numbar, pic); % Add observations (see adddata.m)
    cpTr = adddata(cpTr, cpTs, bpTs, pn, numbar, pic); 
elseif rhm == 'y' && pic == 'p'
    [bprTs, bprTr] = separate(bpr, perc, W); 
    [cprTs, cprTr] = separate(cpr, perc, W);
    
    bprTr = adddata(bprTr, bprTs, cprTs, pn, numbar, pic);
    cprTr = adddata(cprTr, cprTs, bprTs, pn, numbar, pic);
elseif rhm ~= 'y' && pic == 'i'
    [biTs, biTr] = separate(bi, perc, W);
    [ciTs, ciTr] = separate(ci, perc, W); 
    
    biTr = adddata(biTr, biTs, ciTs, in, numbar, pic);
    ciTr = adddata(ciTr, ciTs, biTs, in, numbar, pic); 
elseif rhm == 'y' && pic == 'i',
    [birTs, birTr] = separate(bir, perc, W);
    [cirTs, cirTr] = separate(cir, perc, W);
    
    birTr = adddata(birTr, birTs, cirTs, in, numbar, pic);
    cirTr = adddata(cirTr, cirTs, birTs, in, numbar, pic);
elseif rhm ~= 'y' && pic == 'c'
    [bcTs, bcTr] = separate(bc, perc, W);
    [ccTs, ccTr] = separate(cc, perc, W); 

    bcTr = adddata(bcTr, bcTs, ccTs, cn, numbar, pic);
    ccTr = adddata(ccTr, ccTs, bcTs, cn, numbar, pic);
elseif rhm == 'y' && pic == 'c'
    [bcrTs, bcrTr] = separate(bcr, perc, W); 
    [ccrTs, ccrTr] = separate(ccr, perc, W); 

    bcrTr = adddata(bcrTr, bcrTs, ccrTs, cn, numbar, pic); 
    ccrTr = adddata(ccrTr, ccrTs, bcrTs, cn, numbar, pic);
end

%----------Rename variables---------------%

if (borc == 1) 
	if rhm ~= 'y' && pic == 'p'
        trainit = bpTr; 
    elseif rhm == 'y' && pic == 'p'
        trainit = bprTr; 
    elseif rhm ~= 'y' && pic == 'i'
        trainit = biTr; 
    elseif rhm == 'y' && pic == 'i',
        trainit = birTr; 
    elseif rhm ~= 'y' && pic == 'c'
        trainit = bcTr; 
    elseif rhm == 'y' && pic == 'c'
        trainit = bcrTr; 
	end
else 
    if rhm ~= 'y' && pic == 'p'
        trainit = cpTr;
    elseif rhm == 'y' && pic == 'p'
        trainit = cprTr;
    elseif rhm ~= 'y' && pic == 'i'
        trainit = ciTr;
    elseif rhm == 'y' && pic == 'i',
        trainit = cirTr;
    elseif rhm ~= 'y' && pic == 'c'
        trainit = ccTr;
    elseif rhm == 'y' && pic == 'c'
        trainit = ccrTr;
	end
end

if rhm ~= 'y' && pic == 'p'
    testB = bpTs;
    testC = cpTs;
elseif rhm == 'y' && pic == 'p'
    testB = bprTs;
    testC = cprTs;
elseif rhm ~= 'y' && pic == 'i'
    testB = biTs;
    testC = ciTs;
elseif rhm == 'y' && pic == 'i',
    testB = birTs;
    testC = cirTs;
elseif rhm ~= 'y' && pic == 'c'
    testB = bcTs;
    testC = ccTs;
elseif rhm == 'y' && pic == 'c'
    testB = bcrTs;
    testC = ccrTs;
end

%---------------Model parameters------------------%

if pic == 'p'
    O = 12; % number of output symbols
elseif pic == 'i'
    O = 24;
else
    O = 5;
end

Ln = length(trainit);
Lb1 = L/2;

if numbar == 1
    if pic == 'p'
        Q = Lb1+(Ln-Lb1); % number of states. If a column was added (see adddata.m) increase the number of states by 1.
    else
        Q = Lb1-1+(Ln-Lb1-1);
    end
else
    if pic == 'p'
        Q = L +(Ln-L); 
    else
        Q = L-1+(Ln-(L-1));
    end
end

%----------Make a random guess as to what the model parameters are-----------%
prior0 = normalise(rand(Q,1)); % Inital state distribution 
transmat0 = mk_stochastic(rand(Q,Q)); % state transition probablity distribution %mk_leftright_transmat(Q,p) %mk_stochastic(rand(Q,p))
obsmat0 = mk_stochastic(rand(Q,O)); % observation symbol probablity distribution

transmat2 = HMMtype (transmat0, Q, type); % assign model type (not a function of the HMM Toolbox)

%-----Improve guess of parameters using Baum-Welch reestimation method-----%
% prior2, transmat2, and obsmat2 are the trained model parameters
disp(' ');
disp('training model...');

[LL, prior1, transmat1, obsmat1] = dhmm_em(trainit, prior0, transmat0, obsmat0, 'max_iter', numitr); 

%-----Use the model to compute log likelihood-----%
% log lik is slightly different than LL(end), since it is computed after the final M step
disp(' ');
disp('testing...');

disp('Tested on Bach:'); 
loglik_b = dhmm_logprob(testB, prior1, transmat1, obsmat1)

disp(' ')
disp('Tested on Chopin'); 
loglik_c = dhmm_logprob(testC, prior1, transmat1, obsmat1)
